@@ -1,6 +1,6 @@
 /*
     fileio
-    copyright (c) 1998-2006 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2007 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
     This program is free software; you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
@@ -35,11 +35,11 @@
 struct _FileIO
 {
   gint counter;
+  gchar *file;
 #ifdef G_OS_WIN32
   HANDLE hFile;
 #else /* not G_OS_WIN32 */
   int fd;
-  gchar *file;
   guint share;
 #endif /* not G_OS_WIN32 */
 };
@@ -493,9 +493,9 @@ static volatile gboolean critical = FALSE;
        RET,ファイルポインタ,NULL:エラー                                     */
 FileIO *
 fileio_open (const gchar *file,
-             const guint access,
-             const guint share,
-             const guint mode)
+             const guint  access,
+             const guint  share,
+             const guint  mode)
 {
   gchar *path;
   FileIO *fio;
@@ -533,7 +533,6 @@ fileio_open (const gchar *file,
       g_free (dir);
     }
 #ifdef G_OS_WIN32
-  g_free (path);
 # if GLIB_CHECK_VERSION(2,6,0)
   if (G_WIN32_HAVE_WIDECHAR_API ())
     {
@@ -544,7 +543,10 @@ fileio_open (const gchar *file,
 # if GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE)
       utf8str = g_filename_to_utf8 (file, -1, NULL, NULL, NULL);
       if (!utf8str)
-        return NULL;
+        {
+          g_free (path);
+          return NULL;
+        }
       lpszFile = g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL);
       hFile = CreateFileW (lpszFile, access, share, NULL, mode,
                                                 FILE_ATTRIBUTE_NORMAL, NULL);
@@ -573,9 +575,13 @@ fileio_open (const gchar *file,
     }
 # endif /* GLIB_CHECK_VERSION(2,6,0) */
   if (hFile == INVALID_HANDLE_VALUE)
-    return NULL;
+    {
+      g_free (path);
+      return NULL;
+    }
   fio = g_malloc (sizeof (FileIO));
   fio->counter = 1;
+  fio->file = path;
   fio->hFile = hFile;
 #else /* not G_OS_WIN32 */
   /* ja:フラグ設定 */
@@ -650,8 +656,8 @@ fileio_open (const gchar *file,
     }
   fio = g_malloc (sizeof (FileIO));
   fio->counter = 1;
-  fio->fd = fd;
   fio->file = path;
+  fio->fd = fd;
   fio->share = share;
 # ifdef USE_THREAD
   G_LOCK (critical);
@@ -711,6 +717,7 @@ fileio_unref (FileIO *fio)
       if (fio->counter <= 0)
         {
           result = CloseHandle (fio->hFile);
+          g_free (fio->file);
           g_free (fio);
         }
       else
@@ -747,6 +754,16 @@ fileio_unref (FileIO *fio)
 }
 
 
+/*  ja:ファイルの名前を取得する
+    fio,ファイルポインタ
+    RET,名前(フルパス)                                                      */
+const gchar *
+fileio_getname (FileIO *fio)
+{
+  return (const gchar *)(fio ? fio->file : NULL);
+}
+
+
 /*  ja:ファイルから読み込む
        fio,ファイルポインタ
       data,バッファ
@@ -824,18 +841,14 @@ fileio_seek (FileIO        *fio,
              const guint    whence)
 {
 #ifdef G_OS_WIN32
-# ifdef G_HAVE_GINT64
   LONG lHigh, lLow;
 
   if (!fio)
     return -1;
   lHigh = offset >> 32;
   lLow = SetFilePointer (fio->hFile, (LONG)offset, &lHigh, whence);
-  return lLow != 0xFFFFFFFF || GetLastError () == NO_ERROR
+  return lLow != 0xffffffff || GetLastError () == NO_ERROR
                                 ? ((gint64)lHigh << 32) | (guint32)lLow : -1;
-# else /* not G_HAVE_GINT64 */
-  return SetFilePointer (fio->hFile, offset, NULL, whence);
-# endif /* not G_HAVE_GINT64 */
 #else /* not G_OS_WIN32 */
   gssize result;
 
@@ -861,17 +874,13 @@ goffset
 fileio_size (FileIO *fio)
 {
 #ifdef G_OS_WIN32
-# ifdef G_HAVE_GINT64
   DWORD dwHigh, dwLow;
 
   if (!fio)
     return -1;
   dwLow = GetFileSize (fio->hFile, &dwHigh);
-  return dwLow != 0xFFFFFFFF || GetLastError () == NO_ERROR
+  return dwLow != 0xffffffff || GetLastError () == NO_ERROR
                                 ? ((gint64)dwHigh << 32) | (guint32)dwLow : -1;
-# else /* not G_HAVE_GINT64 */
-  return GetFileSize (fio->hFile, NULL);
-# endif /* not G_HAVE_GINT64 */
 #else /* not G_OS_WIN32 */
   goffset offset, leng;
 
@@ -886,6 +895,40 @@ fileio_size (FileIO *fio)
 }
 
 
+/*  ja:ファイルの終端を設定する
+    fio,ファイルポインタ
+    RET,TRUE:正常終了,FALSE:エラー                                          */
+gboolean
+fileio_eof (FileIO *fio)
+{
+#ifdef G_OS_WIN32
+  return SetEndOfFile (fio->hFile);
+#else /* not G_OS_WIN32 */
+# ifdef HAVE_FTRUNCATE
+  gboolean result;
+  goffset offset;
+
+  offset = fileio_seek (fio, 0, FILEIO_SEEK_CUR);
+  if (offset < 0)
+    return FALSE;
+#  ifdef USE_THREAD
+  G_LOCK (critical);
+  critical = TRUE;
+#  endif /* USE_THREAD */
+  result = fio && g_list_find (glist_fileio, fio)
+                                    ? ftruncate (fio->fd, offset) == 0: FALSE;
+#  ifdef USE_THREAD
+  critical = FALSE;
+  G_UNLOCK (critical);
+#  endif /* USE_THREAD */
+  return result;
+# else /* not HAVE_FTRUNCATE */
+  return FALSE;
+# endif /* not HAVE_FTRUNCATE */
+#endif /* not G_OS_WIN32 */
+}
+
+
 /******************************************************************************
 *                                                                             *
 * ja:高レベルファイル入出力関数群                                             *
@@ -1048,11 +1091,7 @@ fileio_mkdir (const gchar *dir)
   LPTSTR lpszPath;
 #endif /* ! GLIB_CHECK_VERSION(2,6,0) && defined (G_OS_WIN32) */
 
-  if (!dir)
-    return FALSE;
   path = fileio_get_full_path (dir);
-  if (!path)
-    return FALSE;
   parent = g_path_get_dirname (path);
   if (!parent)
     {
@@ -1091,11 +1130,6 @@ fileio_mkdir (const gchar *dir)
 
 # if GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE)
       utf8str = g_filename_to_utf8 (path, -1, NULL, NULL, NULL);
-      if (!utf8str)
-        {
-          g_free (path);
-          return FALSE;
-        }
       lpszPath = g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL);
       g_free (utf8str);
       result = CreateDirectoryW (lpszPath, NULL);
@@ -1136,6 +1170,140 @@ fileio_mkdir (const gchar *dir)
 }
 
 
+/*  ja:ファイル/ディレクトリを削除する
+    path,ファイル/ディレクトリ名
+     RET,TRUE:正常終了,FALSE,エラー                                         */
+gboolean
+fileio_remove (const gchar *path)
+{
+  gboolean result = TRUE;
+
+  if (g_file_test (path, G_FILE_TEST_IS_DIR))
+    {
+      const gchar *name;
+      GDir *dir;
+
+      dir = g_dir_open (path, 0, NULL);
+      if (!dir)
+        return FALSE;
+      while (result && (name = g_dir_read_name (dir)))
+        {
+          gchar *file;
+
+          file = g_build_filename (path, name, NULL);
+          result = fileio_remove (file);
+          g_free (file);
+        }
+      g_dir_close (dir);
+      if (result)
+        {
+#ifdef G_OS_WIN32
+# if ! GLIB_CHECK_VERSION(2,6,0)
+#  ifdef UNICODE
+          gchar *utf8str;
+#  endif /* UNICODE */
+          LPTSTR lpszPath;
+# endif /* not GLIB_CHECK_VERSION(2,6,0) */
+# if GLIB_CHECK_VERSION(2,6,0)
+          if (G_WIN32_HAVE_WIDECHAR_API ())
+            {
+              gchar *utf8str;
+              LPWSTR lpszPath;
+# endif /* GLIB_CHECK_VERSION(2,6,0) */
+
+# if GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE)
+              utf8str = g_filename_to_utf8 (path, -1, NULL, NULL, NULL);
+              lpszPath = g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL);
+              g_free (utf8str);
+              result = RemoveDirectoryW (lpszPath);
+              g_free (lpszPath);
+# endif /* GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE) */
+# if GLIB_CHECK_VERSION(2,6,0)
+            }
+          else
+            {
+              LPSTR lpszPath;
+# endif /* GLIB_CHECK_VERSION(2,6,0) */
+
+# if GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE)
+#  if GLIB_CHECK_VERSION(2,8,0)
+              lpszPath = g_win32_locale_filename_from_utf8 (path);
+#  elif GLIB_CHECK_VERSION(2,6,0)
+              lpszPath = g_locale_from_utf8 (path, -1, NULL, NULL, NULL);
+#  else /* not GLIB_CHECK_VERSION(2,6,0) */
+              lpszPath = g_strdup (path);
+#  endif /* not GLIB_CHECK_VERSION(2,6,0) */
+              result = RemoveDirectoryA (lpszPath);
+              g_free (lpszPath);
+# endif /* GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE) */
+# if GLIB_CHECK_VERSION(2,6,0)
+            }
+# endif /* GLIB_CHECK_VERSION(2,6,0) */
+#else /* not G_OS_WIN32 */
+# if GLIB_CHECK_VERSION(2,6,0)
+          result = g_rmdir (path) == 0;
+# else /* not GLIB_CHECK_VERSION(2,6,0) */
+          result = rmdir (path) == 0;
+# endif /* not GLIB_CHECK_VERSION(2,6,0) */
+#endif /* not G_OS_WIN32 */
+        }
+    }
+  else
+    {
+#ifdef G_OS_WIN32
+# if ! GLIB_CHECK_VERSION(2,6,0)
+#  ifdef UNICODE
+      gchar *utf8str;
+#  endif /* UNICODE */
+      LPTSTR lpszPath;
+# endif /* not GLIB_CHECK_VERSION(2,6,0) */
+# if GLIB_CHECK_VERSION(2,6,0)
+      if (G_WIN32_HAVE_WIDECHAR_API ())
+        {
+          gchar *utf8str;
+          LPWSTR lpszPath;
+# endif /* GLIB_CHECK_VERSION(2,6,0) */
+
+# if GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE)
+          utf8str = g_filename_to_utf8 (path, -1, NULL, NULL, NULL);
+          lpszPath = g_utf8_to_utf16 (utf8str, -1, NULL, NULL, NULL);
+          g_free (utf8str);
+          result = DeleteFileW (lpszPath);
+          g_free (lpszPath);
+# endif /* GLIB_CHECK_VERSION(2,6,0) || defined (UNICODE) */
+# if GLIB_CHECK_VERSION(2,6,0)
+        }
+      else
+        {
+          LPSTR lpszPath;
+# endif /* GLIB_CHECK_VERSION(2,6,0) */
+
+# if GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE)
+#  if GLIB_CHECK_VERSION(2,8,0)
+          lpszPath = g_win32_locale_filename_from_utf8 (path);
+#  elif GLIB_CHECK_VERSION(2,6,0)
+          lpszPath = g_locale_from_utf8 (file, -1, NULL, NULL, NULL);
+#  else /* not GLIB_CHECK_VERSION(2,6,0) */
+          lpszPath = g_strdup (file);
+#  endif /* not GLIB_CHECK_VERSION(2,6,0) */
+          result = DeleteFileA (lpszPath);
+          g_free (lpszPath);
+# endif /* GLIB_CHECK_VERSION(2,6,0) || ! defined (UNICODE) */
+# if GLIB_CHECK_VERSION(2,6,0)
+        }
+# endif /* GLIB_CHECK_VERSION(2,6,0) */
+#else /* not G_OS_WIN32 */
+# if GLIB_CHECK_VERSION(2,6,0)
+      result = g_remove (path) == 0;
+# else /* not GLIB_CHECK_VERSION(2,6,0) */
+      result = remove (path) == 0;
+# endif /* not GLIB_CHECK_VERSION(2,6,0) */
+#endif /* not G_OS_WIN32 */
+    }
+  return result;
+}
+
+
 /******************************************************************************
 *                                                                             *
 * ja:ファイル時刻関数群                                                       *
@@ -1323,7 +1491,7 @@ fileio_settime (const gchar *file,
 }
 
 
-/*  ja:ファイルの時刻を変換する
+/*  ja:システム時刻を取得する
     ftime,ファイルの時刻
     stime,システム時間                                                      */
 void
@@ -1356,6 +1524,42 @@ fileio_getsystime (FileIOTime    *ftime,
 }
 
 
+/*  ja:システム時刻を設定する
+    ftime,ファイルの時刻
+    stime,システム時間                                                      */
+void
+fileio_setsystime (FileIOTime    *ftime,
+                   FileIOSysTime *stime)
+{
+#ifdef G_OS_WIN32
+  SYSTEMTIME st;
+
+  st.wYear         = stime->year;
+  st.wMonth        = stime->mon;
+  st.wDayOfWeek    = stime->wday;
+  st.wDay          = stime->mday;
+  st.wHour         = stime->hour;
+  st.wMinute       = stime->min;
+  st.wSecond       = stime->sec;
+  st.wMilliseconds = 0;
+  SystemTimeToFileTime (&st, ftime);
+#else /* not G_OS_WIN32 */
+  struct tm gmt;
+
+  gmt.tm_sec   = stime->sec;
+  gmt.tm_min   = stime->min;
+  gmt.tm_hour  = stime->hour;
+  gmt.tm_mday  = stime->mday;
+  gmt.tm_mon   = stime->mon - 1;
+  gmt.tm_year  = stime->year - 1900;
+  gmt.tm_wday  = stime->wday;
+  gmt.tm_yday  = 0;
+  gmt.tm_isdst = -1;
+  *ftime = mktime (&gmt);
+#endif /* not G_OS_WIN32 */
+}
+
+
 /******************************************************************************
 *                                                                             *
 * ja:応用ファイル関数群                                                       *
@@ -1498,7 +1702,7 @@ fileio_extract_icon (const gchar *file,
                     alpha = TRUE;
                   pixels += 4;
                 }
-              pixels += (w * 4 - rowstride);
+              pixels += rowstride - w * 4;
             }
           /* en:mask */
           if (!alpha)
@@ -1513,7 +1717,7 @@ fileio_extract_icon (const gchar *file,
                       pixels[3] = 255 - lpbBits[(x + y * w) * 4];
                       pixels += 4;
                     }
-                  pixels += (w * 4 - rowstride);
+                  pixels += rowstride - w * 4;
                 }
             }
           g_free (lpbBits);
