@@ -1,10 +1,10 @@
 /*
     orzwinmenu
-    copyright (c) 1998-2006 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
+    copyright (c) 1998-2008 Kazuki IWAMOTO http://www.maid.org/ iwm@maid.org
 
-    This program is free software; you can redistribute it and/or modify
+    This program is free software: you can redistribute it and/or modify
     it under the terms of the GNU General Public License as published by
-    the Free Software Foundation; either version 2 of the License, or
+    the Free Software Foundation, either version 3 of the License, or
     (at your option) any later version.
 
     This program is distributed in the hope that it will be useful,
@@ -13,48 +13,16 @@
     GNU General Public License for more details.
 
     You should have received a copy of the GNU General Public License
-    along with this program; if not, write to the Free Software
-    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
+    along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
 #include "orzwinmenu.h"
 
 
-static void orz_winmenu_class_init (OrzWinmenuClass *klass);
-static void orz_winmenu_init       (OrzWinmenu      *winmenu);
-static void orz_winmenu_dispose    (GObject         *object);
-
-
-static GObjectClass *parent_class = NULL;
-
-
 /******************************************************************************
 *                                                                             *
 ******************************************************************************/
-GType
-orz_winmenu_get_type (void)
-{
-  static GType type = 0;
-
-  if (!type)
-    {
-      const static GTypeInfo info =
-      {
-        sizeof (OrzWinmenuClass),
-        NULL,               /* base_init */
-        NULL,               /* base_finalize */
-        (GClassInitFunc)orz_winmenu_class_init,
-        NULL,               /* class_finalize */
-        NULL,               /* class_data */
-        sizeof (OrzWinmenu),
-        0,              /* n_preallocs */
-        (GInstanceInitFunc)orz_winmenu_init,
-      };
-
-      type = g_type_register_static (G_TYPE_OBJECT, "OrzWinmenu", &info, 0);
-    }
-
-  return type;
-}
+G_DEFINE_TYPE (OrzWinmenu, orz_winmenu, G_TYPE_OBJECT);
+static void orz_winmenu_dispose (GObject *object);
 
 
 static void
@@ -62,9 +30,7 @@ orz_winmenu_class_init (OrzWinmenuClass *klass)
 {
   GObjectClass *object_class;
 
-  parent_class = g_type_class_peek_parent (klass);
-  object_class = (GObjectClass *) klass;
-
+  object_class = G_OBJECT_CLASS (klass);
   object_class->dispose = orz_winmenu_dispose;
 }
 
@@ -377,8 +343,9 @@ orz_winmenu_switch (OrzMdi     *mdi,
 static void
 orz_winmenu_dispose (GObject *object)
 {
-  OrzWinmenu *winmenu = ORZ_WINMENU (object);
+  OrzWinmenu *winmenu;
 
+  winmenu = ORZ_WINMENU (object);
   if (winmenu->mdi)
     {
       gint i;
@@ -412,8 +379,8 @@ orz_winmenu_dispose (GObject *object)
                                                 winmenu->handler_destroy_menu);
       winmenu->menu_shell = NULL;
     }
-  if (G_OBJECT_CLASS (parent_class)->dispose)
-    G_OBJECT_CLASS (parent_class)->dispose (object);
+  if (G_OBJECT_CLASS (orz_winmenu_parent_class)->dispose)
+    G_OBJECT_CLASS (orz_winmenu_parent_class)->dispose (object);
 }
 
 
@@ -457,7 +424,7 @@ orz_winmenu_destroy_menu_shell (GtkWidget  *widget,
 ******************************************************************************/
 /*  ja:新規作成
     RET,オブジェクト                                                        */
-GObject*
+GObject *
 orz_winmenu_new (void)
 {
   return g_object_new (ORZ_TYPE_WINMENU, NULL);
@@ -497,6 +464,16 @@ orz_winmenu_destroy_menu (OrzWinmenu *winmenu)
 }
 
 
+/*  ja:MDIを取得する
+    winmenu,オブジェクト
+        RET,MDI                                                             */
+OrzMdi *
+orz_winmenu_get_mdi (OrzWinmenu *winmenu)
+{
+  return winmenu ? winmenu->mdi : NULL;
+}
+
+
 /*  ja:MDIを設定する
     winmenu,オブジェクト
        mdi,MDI                                                              */
@@ -567,6 +544,16 @@ orz_winmenu_set_mdi (OrzWinmenu *winmenu,
 }
 
 
+/*  ja:メニューを取得する
+    winmenu,オブジェクト
+        RET,メニュー                                                        */
+GtkWidget *
+orz_winmenu_get_menu (OrzWinmenu *winmenu)
+{
+  return winmenu ? winmenu->menu_shell : NULL;
+}
+
+
 /*  ja:メニューを設定する
        winmenu,オブジェクト
     menu_shell,メニュー
